
/*
 * Copyright (c) Abraham vd Merwe <abz@blio.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *	  notice, this list of conditions and the following disclaimer in the
 *	  documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of other contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>

#include <tinysnmp/tinysnmp.h>
#include <tinysnmp/agent/mod.h>
#include <tinysnmp/agent/odb.h>
#include <tinysnmp/compat/inttypes.h>

#include <ber/ber.h>

#include <debug/log.h>
#include <debug/memory.h>

typedef struct
{
   uint32_t *oid;
   snmp_value_t value;
   uint32_t counter;
} t1_t;

static t1_t t1_private;

static int t1_open (const struct mod_r *mod,const char *confdir)
{
   if ((t1_private.oid = make_oid ("1.2.3.4")) == NULL)
	 {
		log_printf (LOG_ERROR,"Couldn't create ObjectID 1.2.3.4\n");
		return (-1);
	 }

   t1_private.value.type = BER_Counter32;
   t1_private.value.data.Counter32 = 0;

   return (0);
}

static int t1_update (const struct mod_r *mod)
{
   t1_t *priv = &t1_private;

   odb_remove (priv->oid);

   if (odb_add (priv->oid,&priv->value) < 0)
	 {
		log_printf (LOG_WARNING,"Couldn't add ObjectID 1.2.3.4\n");
		return (-1);
	 }

   priv->value.data.Counter32++;

   return (0);
}

static void t1_close (const struct mod_r *mod)
{
   t1_t *priv = &t1_private;
   mem_free (priv->oid);
}

static int t2_update (const struct mod_r *mod)
{
   static const uint32_t oid[] = { 3, 42, 4, 0 };
   snmp_value_t value;

   odb_remove (oid);

   value.type = BER_INTEGER;
   value.data.INTEGER = 314159;

   if (odb_add (oid,&value) < 0)
	 {
		log_printf (LOG_WARNING,"Couldn't add ObjectID 1.2.4.0\n");
		return (-1);
	 }

   return (0);
}

static int t3_open (const struct mod_r *mod,const char *confdir)
{
   return (0);
}

static int t3_update (const struct mod_r *mod)
{
   return (0);
}

static void t3_close (const struct mod_r *mod)
{
}

static const uint32_t t1_oid[3] = { 2, 42, 3 };
static const uint32_t t2_oid[3] = { 2, 42, 4 };
static const uint32_t t3_oid[3] = { 2, 42, 7 };
static const uint32_t t4_oid[3] = { 2, 42, 8 };

const mod_t mod[] =
{
   { "test/1", t1_open, t1_update, t1_close, t1_oid },
   { "test/2", NULL, t2_update, NULL, t2_oid },
   { "test/3", t3_open, t3_update, NULL, t3_oid },
   { "test/4", NULL, t3_update, t3_close, t4_oid },
   { NULL, NULL, NULL, NULL, NULL }
};

